from .crud_datasets import datasets
from .crud_docker_trains import docker_trains
from .crud_fhir_servers import fhir_servers
from .crud_local_train import local_train
from .crud_notifications import notifications

# from .crud_discovery import discoveries
# TODO cleanup namespace inconsistencies
